#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.6.1
 Author:         Matwachich

 Script Function:
	GEngin - Sprites

#ce ----------------------------------------------------------------------------

Global Const $__GEng_SpriteArrayUB = 9

; Constantes d'origine
Global Enum $GEng_Middle, $GEng_TopLeft, $GEng_TopRight, $GEng_BotLeft, $GEng_BotRight

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteCreate
; Description....:	Crer un objet sprite
; Parameters.....:	Rien
; Return values..:	L'objet sprite
; Author.........:	Matwachich
; Remarks........:	Pour supprimer un sprite, rinitialisez le  0
; ===========================================================================================================
Func _GEng_SpriteCreate()
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	Local $ret[$__GEng_SpriteArrayUB]
	__GEng_SpriteArrayInit($ret)
	Return $ret
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteSetImg
; Description....:	Charge un objet image dans l'objet sprite spcifi
; Parameters.....:	$hSprite = L'objet sprite  modifier
;					$hImg = Objet image  charger dans l'objet sprite
;						Si Par defaut Alors: rinitialise l'objet sprite (aucun image)
; Return values..:	Succes: 1
;					Erreur: 0 et
;						@error = 1 - Objet sprite incorrect
;						@error = 2 - Objet image incorrect
; Author.........:	Matwachich
; Remarks........:	Cette fonction met automatiquement les paramtres taille de l'objet sprite
;						aux dimension de l'image
; ===========================================================================================================
Func _GEng_SpriteSetImg(ByRef $hSprite, $hImg = -1)
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	; ---
	If $hImg = -1 Then
		__GEng_SpriteArrayInit($hSprite)
	EndIf
	; ---
	If Not __GEng_IsImg($hImg) Then Return SetError(2, 0, 0)
	; ---
	$hSprite[0] = $hImg
	; ---
	$hSprite[3] = __GEng_hImg_GetDimensions($hImg, "x")
	$hSprite[4] = __GEng_hImg_GetDimensions($hImg, "y")
	; ---
	Return SetError(0, 0, 1)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteSet
; Description....:	Modifie les paramtres coordones, taille et origine d'un objet sprite
; Parameters.....:	$hSprite = Objet sprite  modifier
;					$iX, $iY = Paramtres Coordones
;					$iW, $iH = Paramtres Taille
;						Si par defaut Alors: les paramtres ne sont pas modifis
;						Si = -1 Alors: Les paramtres son rinitialiss aux dimensions de l'image actuellement
;										 charg dans l'objet sprite.
;					$eOrigin = Dfini le point d'origine du sprite, peut tre:
;						- $GEng_Middle = milieu
;						- $GEng_TopLeft = Coin suprieur gauche
;						- $GEng_TopRight = Coin suprieur droit
;						- $GEng_BotLeft = Coin infrieur gauche
;						- $GEng_BotRight = Coin infrieur droit
;					$originX, $originY = Dfini manuellement le point d'origine du sprite.
;						Il faut que ces 2 paramtres soient renseigns pour que $eOrigin soit ignor.
; Return values..:	Succes: 1
;					Erreur: 0 Et
;						@error = 1 - Objet sprite incorrect
;						@error = 2 - Aucune image charg dans l'objet sprite
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _GEng_SpriteSet(ByRef $hSprite, $iX, $iY, $iW = Default, $iH = Default, $eOrigin = 0, $originX = Default, $originY = Default)
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	; ---
	__GEng_SpriteSetSize($hSprite, $iW, $iH)
	; ---
	; On calcule l'origine apres avoir mis en place les dimensions de l'image
	; dans le bloc prcdent, pour que les macros d'origine ($GEng_Middle, $GEng_TopLeft...)
	; soient correctes
	__GEng_SpriteSetOrigin($hSprite, $eOrigin, $originX, $originY)
	$hSprite[1] = $iX - $hSprite[5] ; position donne en paramtre - origine
	$hSprite[2] = $iY - $hSprite[6] ;
	; ---
	Return SetError(0, 0, 1)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteDraw
; Description....:	Dssine un objet sprite  l'cran
; Parameters.....:	$hSprite = Objet sprite  dssiner
; Return values..:	Succes: 1
;					Erreur: 0 Et
;						@error = 1 - Objet sprite incorrect
;						@error = 2 - Aucune image charg dans l'objet sprite
;						@error = 3 - Paramtres de l'objet sprite incorrects
; Author.........:	Matwachich
; Remarks........:	Pour valider l'affichage d'un sprite, il faut appeler _GEng_DrawEnd()
; ===========================================================================================================
Func _GEng_SpriteDraw($hSprite)
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	If $hSprite[1] = -1 Or $hSprite[2] = -1 Or $hSprite[3] <= 0 Or $hSprite[4] <= 0 Then
		Return SetError(3, 0, 0)
	EndIf
	; ---
	Local $spr_hImg = $hSprite[0]
	Local $spr_X = $hSprite[1]
	Local $spr_Y = $hSprite[2]
	Local $spr_W = $hSprite[3]
	Local $spr_H = $hSprite[4]
	; ---
	Return _GDIPlus_GraphicsDrawImageRect($__GEng_MainArray[3], __GEng_hImg_GetHandle($spr_hImg), _
			$spr_X, $spr_Y, _
			$spr_W, $spr_H)
EndFunc

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteGetPos, _GEng_SpriteGetSize, _GEng_SpriteGetOrigin
; Description....:	Retourne les paramtres du sprite $hSprite
; Parameters.....:	$hSprite = Objet sprite  intrroger
; Return values..:	Tous les retours sont de forme:
;						$array[2] = [Paramtre X, Paramtre Y]
;						(Similaire  MouseGetPos)
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _GEng_SpriteGetPos(ByRef $hSprite) ; retourne la position du point d'origine par rapport  l'cran
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	; ---
	Local $ret[2] = [$hSprite[1] + $hSprite[5], $hSprite[2] + $hSprite[6]]
	Return $ret
EndFunc

; ##############################################################

Func _GEng_SpriteGetSize(ByRef $hSprite) ; retourne la taille du sprite
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	; ---
	Local $ret[2] = [$hSprite[3], $hSprite[4]]
	Return $ret
EndFunc

; ##############################################################

Func _GEng_SpriteGetOrigin(ByRef $hSprite) ; retourne la position du point d'origine dans le sprite
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	; ---
	Local $ret[2] = [$hSprite[5], $hSprite[6]]
	Return $ret
EndFunc





; ==============================================================
; ### Internals
; ==============================================================
Func __GEng_SpriteArrayInit(ByRef $hSprite)
	$hSprite[0] = -1 ; hImg index in $__GEng_LoadedImg[]
	$hSprite[1] = -1 ; pos X
	$hSprite[2] = -1 ; pos Y
	$hSprite[3] = -1 ; size X (width)
	$hSprite[4] = -1 ; size Y (height)
	$hSprite[5] = 0  ; origin X
	$hSprite[6] = 0  ; origin Y
	$hSprite[7] = 0  ; anim object
	$hSprite[8] = 0  ; anim frame
EndFunc

Func __GEng_IsSprite(ByRef $hSprite)
	If UBound($hSprite) <> $__GEng_SpriteArrayUB Then Return 0
	Return 1
EndFunc

Func __GEng_SpriteSetSize(ByRef $hSprite, $iW = Default, $iH = Default)
	If $iW <> Default Then
		If $iW <> -1 Then
			$hSprite[3] = $iW
		Else
			$hSprite[3] = __GEng_hImg_GetDimensions($hSprite[0], 'x')
		EndIf
	EndIf
	If $iH <> Default Then
		If $iH <> -1 Then
			$hSprite[4] = $iH
		Else
			$hSprite[4] = __GEng_hImg_GetDimensions($hSprite[0], 'y')
		EndIf
	EndIf
EndFunc

Func __GEng_SpriteSetOrigin(ByRef $hSprite, $iPos, $iX = Default, $iY = Default)
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite) Then Return SetError(1, 0, 0)
	If $hSprite[0] = -1 Then Return SetError(2, 0, 0)
	; ---
	Local $ret = 1, $err = 0
	If $iX <> Default And $iY <> Default Then
		$hSprite[5] = $iX
		$hSprite[6] = $iY
	Else
		Switch $iPos
			Case $GEng_Middle
				$hSprite[5] = $hSprite[3] / 2
				$hSprite[6] = $hSprite[4] / 2
			Case $GEng_TopLeft
				$hSprite[5] = 0
				$hSprite[6] = 0
			Case $GEng_TopRight
				$hSprite[5] = $hSprite[3]
				$hSprite[6] = 0
			Case $GEng_BotLeft
				$hSprite[5] = 0
				$hSprite[6] = $hSprite[4]
			Case $GEng_BotRight
				$hSprite[5] = $hSprite[3]
				$hSprite[6] = $hSprite[4]
			Case Else
				$ret = 0
				$err = 1
		EndSwitch
	EndIf
	; ---
	Return SetError($err, 0, $ret)
EndFunc
