#cs ----------------------------------------------------------------------------

 AutoIt Version: 3.3.6.1
 Author:         Matwachich

 Script Function:
	GEngin - Sprite collision

#ce ----------------------------------------------------------------------------

; pour les tests de collision avec les bord de la fentre
Global Enum $GEng_BorderTop, $GEng_BorderBot, $GEng_BorderLeft, $GEng_BorderRight

; # FUNCTION # ==============================================================================================
; Name...........:	_GEng_SpriteCollision
; Description....:	Test si il y a collision entre les 2 sprites en paramtres
; Parameters.....:	$hSprite1, $hSprite2 = Les sprites  tester
;					$iCorrection1, $iCorrection2 = Seuil de tolrance, en nombre de pixels, appliqu aux
;						Objets Sprite dans l'ordre
; Return values..:	1 - Collision
;					0 - Pas de collision
;					Si @error = 1 - Un des 2 paramtres n'est pas un sprite valide
; Author.........:	Matwachich
; Remarks........:	
; ===========================================================================================================
Func _GEng_SpriteCollision($hSprite1, $hSprite2, $iCorrection1X = 0, $iCorrection1Y = 0, _
												 $iCorrection2X = 0, $iCorrection2Y = 0)
	If Not _GEng_IsStarted() Then Return SetError(-1, 0, 0)
	; ---
	If Not __GEng_IsSprite($hSprite1) Then Return SetError(1, 0, 0)
	If Not __GEng_IsSprite($hSprite2) Then Return SetError(1, 0, 0)
	; ---
	Local $pos1, $pos2, $size1, $size2, $ori1, $ori2, $collision = 0
	$pos1 = _GEng_SpriteGetPos($hSprite1)
	$pos2 = _GEng_SpriteGetPos($hSprite2)
	$size1 = _GEng_SpriteGetSize($hSprite1)
	$size2 = _GEng_SpriteGetSize($hSprite2)
	$ori1 = _GEng_SpriteGetOrigin($hSprite1)
	$ori2 = _GEng_SpriteGetOrigin($hSprite2)
	; --- correction du point d'origine
	$pos1[0] -= $ori1[0]
	$pos1[1] -= $ori1[1]
	$pos2[0] -= $ori2[0]
	$pos2[1] -= $ori2[1]
	; --- application des paramtres $iCorrection
	$pos1[0] += $iCorrection1X
	$pos1[1] += $iCorrection1Y
	$size1[0] -= 2 * $iCorrection1X
	$size1[1] -= 2 * $iCorrection1Y
	$pos2[0] += $iCorrection2X
	$pos2[1] += $iCorrection2Y
	$size2[0] -= 2 * $iCorrection2X
	$size2[1] -= 2 * $iCorrection2Y
	; --- Test du point infrieur droit de spr1 OK
	If ($pos1[0] + $size1[0]) > $pos2[0] And ($pos1[0] + $size1[0]) < ($pos2[0] + $size2[0]) And _
	   ($pos1[1] + $size1[1]) > $pos2[1] And ($pos1[1] + $size1[1]) < ($pos2[1] + $size2[1]) Then
		; ---
		$collision = 1
	EndIf
	; --- Test du point suprieur gauche de spr1 OK
	If $pos1[0] > $pos2[0] And $pos1[0] < ($pos2[0] + $size2[0]) And _
	   $pos1[1] > $pos2[1] And $pos1[1] < ($pos2[1] + $size2[1]) Then
		; ---
		$collision = 1
	EndIf
	; --- Test du point infrieur gauche de spr1 OK
	If $pos1[0] > $pos2[0] And $pos1[0] < ($pos2[0] + $size2[0]) And _
	   ($pos1[1] + $size1[1]) > $pos2[1] And ($pos1[1] + $size1[1]) < ($pos2[1] + $size2[1]) Then
		; ---
		$collision = 1
	EndIf
	; --- Test du point suprieur droit de spr 1
	If ($pos1[0] + $size1[0]) > $pos2[0] And ($pos1[0] + $size1[0]) < ($pos2[0] + $size2[0]) And _
	    $pos1[1] > $pos2[1] And $pos1[1] < ($pos2[1] + $size2[1]) Then
		; ---
		$collision = 1
	EndIf
	; --- DEBUG
	If $collision Then
		If $__GEng_Debug Then
			_GDIPlus_GraphicsDrawRect($__GEng_MainArray[1], $pos1[0], $pos1[1], $size1[0], $size1[1], $_dbg_pen1)
			_GDIPlus_GraphicsDrawRect($__GEng_MainArray[1], $pos2[0], $pos2[1], $size2[0], $size2[1], $_dbg_pen2)
			Sleep(50)
		EndIf
	EndIf
	; ---
	Return $collision
EndFunc

